/*
 * Copyright 2010 Sergey Martynov http://studiomobile.ru
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.studiomobile.android.http;

import org.apache.http.*;
import org.apache.http.client.methods.*;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.EntityEnclosingRequestWrapper;
import org.apache.http.impl.client.RequestWrapper;
import org.apache.http.nio.reactor.IOReactorStatus;
import org.apache.http.params.HttpParams;
import ru.studiomobile.http.HttpClient;
import ru.studiomobile.http.HttpClientHelper;
import ru.studiomobile.http.HttpRequestFuture;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.URL;

public class AndroidHttpClient extends HttpClientHelper implements HttpClient {
    private final org.apache.http.client.HttpClient client;

    public AndroidHttpClient(org.apache.http.client.HttpClient client) {
        this.client = client;
    }

    public AndroidHttpClient(HttpParams config) throws IOException {
        this(new DefaultHttpClient(config));
    }

    public AndroidHttpClient() throws IOException {
        this(new DefaultHttpClient());
    }


    private HttpRequestFuture execute(HttpUriRequest request, Header... headers) {
        for (Header header : headers) {
            request.addHeader(header);
        }
        AndroidHttpRequestFuture future = new AndroidHttpRequestFuture(client, request);
        future.execute();
        return future;
    }

    public HttpRequestFuture executeRequest(InetSocketAddress address, HttpRequest request) {
        try {
            return execute(wrapRequest(request));
        } catch (ProtocolException e) {
            throw new RuntimeException(e);
        }
    }

    private HttpUriRequest wrapRequest(HttpRequest request) throws ProtocolException {
        if (request instanceof HttpEntityEnclosingRequest) {
            return new EntityEnclosingRequestWrapper((HttpEntityEnclosingRequest) request);
        }
        return new RequestWrapper(request);
    }

    public HttpRequestFuture head(URL url, Header... headers) {
        return execute(new HttpHead(url.toExternalForm()), headers);
    }

    public HttpRequestFuture get(URL url, Header... headers) {
        return execute(new HttpGet(url.toExternalForm()), headers);
    }

    public HttpRequestFuture post(URL url, HttpEntity entity, Header... headers) {
        HttpPost post = new HttpPost(url.toExternalForm());
        post.setEntity(entity);
        return execute(post, headers);
    }

    public HttpRequestFuture put(URL url, HttpEntity entity, Header... headers) {
        HttpPut put = new HttpPut(url.toExternalForm());
        put.setEntity(entity);
        return execute(put, headers);
    }

    public HttpRequestFuture delete(URL url, Header... headers) {
        return execute(new HttpDelete(url.toExternalForm()), headers);
    }

    public IOReactorStatus getStatus() {
        return IOReactorStatus.ACTIVE;
    }

    public void shutdown(long waitMs) throws IOException {
    }

    public void shutdown() throws IOException {
    }
}
