package ru.studiomobile.android.http;

import android.os.AsyncTask;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import ru.studiomobile.http.HttpClientListener;
import ru.studiomobile.http.HttpRequestFuture;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

public class AndroidHttpRequestFuture implements HttpRequestFuture, AsyncRequestTask.Listener {

    private final AsyncRequestTask task;
    private HttpClientListener listener;

    public AndroidHttpRequestFuture(HttpClient client, HttpUriRequest request) {
        task = new AsyncRequestTask(client, request);
        task.setListener(this);
    }

    public void execute() {
        task.execute();
    }

    public HttpRequest getRequest() {
        return task.getRequest();
    }

    public Iterable<HttpRequest> getRedirects() {
        return null;
    }

    public Exception getError() {
        return task.getError();
    }

    public HttpClientListener getListener() {
        return listener;
    }

    public void setListener(HttpClientListener listener) {
        this.listener = listener;
        if (listener == null) return;
        if (isDone()) {
            if (isCancelled()) {
                listener.requestCancelled(this);
                return;
            }
            if (task.getError() != null) {
                listener.requestFailed(this);
                return;
            }
            listener.requestFinished(this, task.getResponse());
        }
    }

    public boolean cancel(boolean mayInterrupt) {
        boolean canceled = task.cancel(mayInterrupt);
        if (listener != null) {
            listener.requestCancelled(this);
        }
        return canceled;
    }

    public boolean isCancelled() {
        return task.isCancelled();
    }

    public boolean isDone() {
        return task.getStatus() == AsyncTask.Status.FINISHED;
    }

    public HttpResponse get() throws InterruptedException, ExecutionException {
        return task.get();
    }

    public HttpResponse get(long l, TimeUnit timeUnit) throws InterruptedException, ExecutionException, TimeoutException {
        return task.get(l, timeUnit);
    }

    public void taskFinished(HttpResponse response) {
        if (listener != null) {
            listener.requestFinished(this, response);
        }
    }

    public void taskFailed(Exception error) {
        if (listener != null) {
            listener.requestFailed(this);
        }
    }
}
