package ru.studiomobile.android.http;

import android.os.AsyncTask;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;

import java.io.IOException;

public class AsyncRequestTask extends AsyncTask<Integer, Integer, HttpResponse> {

    public interface Listener {
        public void taskFinished(HttpResponse response);
        public void taskFailed(Exception error);
    }

    private final HttpClient client;
    private final HttpUriRequest request;
    private volatile HttpResponse response;
    private volatile Exception error;
    private Listener listener;

    public AsyncRequestTask(HttpClient client, HttpUriRequest request) {
        this.client = client;
        this.request = request;
    }

    public HttpUriRequest getRequest() {
        return request;
    }

    public HttpResponse getResponse() {
        return response;
    }

    public Exception getError() {
        return error;
    }

    public Listener getListener() {
        return listener;
    }

    public void setListener(Listener listener) {
        this.listener = listener;
    }

    @Override
    protected HttpResponse doInBackground(Integer... params) {
        try {
            return response = client.execute(request);
        } catch (IOException e) {
            error = e;
        }
        return null;
    }

    @Override
    protected void onPostExecute(HttpResponse response) {
        if (listener == null) return;
        if (error == null) {
            listener.taskFinished(response);
        } else {
            listener.taskFailed(error);
        }
    }
}
